#!/usr/bin/env bash

SCRIPTDIR=$(dirname $0)
OUTDIR=.
#Hard-coded location
ETC_UPDATE_URL="http://www.ece.purdue.edu/~zhang97/ispy/ispy_etc_release.tar.gz"

Usage () {
    echo "./super.sh (optional:) -p para -S source_ip -o outdir"
}

Prepare_list() {
    CACHEDIR=$OUTDIR
    TMPDIR=preparelist.tmp
    mkdir -p $TMPDIR
    for method in trace ping; do
        awk -F'|' '$3<2' $CACHEDIR/liveip.${method}.cache  | sort -t'|' -k 2,2 \
            | awk -F'|' 'BEGIN{x=0} {if (x!=$2) { if (x!=0) {printf "\n";} printf "%s|%s",$2,$1;} else {printf " %s",$1;} x=$2} END{printf "\n"}' \
            | sort -k1,1 -t '|' >$TMPDIR/${method}.ips
    done
    join -t '|' $TMPDIR/trace.ips $TMPDIR/ping.ips \
        > $OUTDIR/probe.ips
}

Check_for_Update() {
    #Check for data update
    echo "Check for update `date`"
    local etc_file=$(basename $ETC_UPDATE_URL)
    wget -O $OUTDIR/$etc_file $ETC_UPDATE_URL
    echo "Extracting Ip2Prefix mapping ..."
    tar zxf $OUTDIR/$etc_file -C $OUTDIR
    mv $OUTDIR/ispy_etc_release/* $OUTDIR/
    test -s $OUTDIR/liveip.trace.cache || \
        { echo "Bad update. Quit."; exit 1; }
    Prepare_list

#Was
#     rsync -av --rsh="$sshopt" \
#         $ETC_UPDATE_URL/* \
#         $OUTDIR/; then
#         echo "Extracting Ip2Prefix mapping ..."
#         (cd $OUTDIR; \
#             gzip -f -c -d PrefixToAS.map.gz > PrefixToAS.map; \
#             rm -rf PrefixToAS.map.partitioned; \
#             tar zxf PrefixToAS.map.partitioned.tar.gz)
#         test -s $OUTDIR/liveip.trace.cache || \
#             { echo "Bad update. Quit."; exit 1; }
    Prepare_list
}

while getopts "o:p:S:" flag
do
    case $flag in
        o)  OUTDIR=$OPTARG;;
        p)  PARA="-p $OPTARG";;
        S)  SRCIP_OPT="-S $OPTARG";;
        *)  Usage; exit 1;;
    esac
done

test -n "$OUTDIR" || {  Usage; exit 1; }

test -d $OUTDIR || mkdir $OUTDIR
test -d $OUTDIR/finished || mkdir $OUTDIR/finished

echo $$ > $OUTDIR/super.pid

Check_for_Update

rm -rf $OUTDIR/ongoing.tmp/
rm -rf $OUTDIR/ongoing.out/
mkdir $OUTDIR/ongoing.out
mkdir $OUTDIR/ongoing.tmp

#Start mini httpd for display
$SCRIPTDIR/../mini_httpd-1.19/mini_httpd -p 8900 \
    -i $OUTDIR/mini_httpd.pid -d $OUTDIR -D \
    2> $OUTDIR/mini_httpd.err > $OUTDIR/mini_httpd.out &

$SCRIPTDIR/../display/display.sh $OUTDIR/ongoing.out/probe.log $OUTDIR/graph/ \
    2> $OUTDIR/display.err > $OUTDIR/display.out &
echo $! > $OUTDIR/display.pid

$SCRIPTDIR/../probing/dProbe.pl $PARA \
    -M $OUTDIR/PrefixToAS.map \
    -o $OUTDIR/ongoing.out \
    -t $OUTDIR/ongoing.tmp \
    $SRCIP_OPT $OUTDIR/probe.ips

