#!/usr/bin/env bash

######################################################
# iSPY Project                                       #
# Copyright (c) 2008, Zheng Zhang, Purdue University #
######################################################

test -n "$2" || { echo >&2 "Usage: ./display.sh input_log outdir"; exit 1; }

INTERVAL=10; #sec
SIZE=60;
RANGE=$((INTERVAL*SIZE/60))
INPUT=$1
OUTDIR=$2
mkdir -p $OUTDIR

plot() {
    gnuplot<<EOF
set terminal postscript eps color
set output "$OUTDIR/unreach.new.eps"
set size 1, 0.4
set xrange [-$RANGE:0]
set title "Unreachable ASes"
set ylabel "Unreachable ASes"
set xlabel "Time (past 10 minutes)"
set yrange [0:4000]
plot "$OUTDIR/stat.pd" using 1:2 notitle w lp 5
EOF

    gnuplot<<EOF
set terminal postscript eps color
set output "$OUTDIR/cut.new.eps"
set size 1, 0.4
set xrange [-$RANGE:0]
set title "|Cuts|"
set ylabel "|Cuts|"
set xlabel "Time (past 10 minutes)"
set yrange [0:100]
plot "$OUTDIR/stat.pd" using 1:3 title "lower bound" w lp 4, \
     "$OUTDIR/stat.pd" using 1:4 title "upper bound" w lp 9, \
     10 with lines lt 1 notitle
EOF
}

render_page() {
    eps2png -f -m 256 -d 150 $OUTDIR/cut.new.eps
    eps2png -f -m 256 -d 150 $OUTDIR/unreach.new.eps

    cat>$OUTDIR/index.html.mv<<EOF
<HTML>
<META http-equiv="refresh" content="10"> 
<TITLE>Prefix Hijacking Demo</TITLE>
<BODY>
<CENTER>
$ALARMSTR
<P>
<B>Current time = $now</B>
<P>
<IMG src="unreach.png"></IMG>
<P>
<IMG src="cut.png"></IMG>
</CENTER>
</BODY>
</HTML>
EOF
    
    mv $OUTDIR/index.html.mv $OUTDIR/index.html
}


echo -ne '' > $OUTDIR/stat
while true; do
    tail -n 2 $INPUT | head -n 1 \
        | sed 's/^\([0-9\.]*\).*nUnreach=\(.*\) nCut=\[\(.*\),\(.*\)\] alarm=\(.*\)$/\1 \2 \3 \4 \5/g' \
        > $OUTDIR/stat.newline
    
    cat $OUTDIR/stat.newline $OUTDIR/stat | head -n $SIZE > $OUTDIR/stat.mv
    cp $OUTDIR/stat.mv $OUTDIR/stat
    awk '{print -NR*'$INTERVAL'/60,$2,$3,$4}' $OUTDIR/stat > $OUTDIR/stat.pd

    alarm=$(awk '{print $5}' $OUTDIR/stat.newline)
    if test "$alarm" == "1"; then
        ALARMSTR='<p><B><font color=red size="+4">Our prefix is hijacked!</font></B></p>'
    else
        ALARMSTR='';
    fi
    
    now=$(date)
    
    plot
    render_page

    mv $OUTDIR/unreach.new.png $OUTDIR/unreach.png
    mv $OUTDIR/cut.new.png $OUTDIR/cut.png
    
    echo "wait a while"
    sleep $INTERVAL
done

