/*************************************************************/
/* iTraceroute (part of iSPY) Project                        */
/* Copyright (c) 2008, Zheng Zhang, Purdue University        */
/*************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#include <iostream>
#include <sstream>
#include <string>
using namespace std;

#include "NetUtil.h"
#include "PP_IcmpEcho.h"


void
PP_IcmpEcho::
create_raw_socket (int *sockd)
{
    int on = 1;
    
    int proto = IPPROTO_ICMP;
#if defined __OS_CYGWIN__ || defined __OS_BSD__
    proto = IPPROTO_IP;
#elif defined __OS_LINUX__
    ;
#else
#error "NO OS"
#endif

    if((*sockd = socket(PF_INET, SOCK_RAW, proto)) < 0)  {//was IPPROTO_TCP
        throw MyException (str_log(ERROR, "socket"));
    }
    
    if(setsockopt(*sockd, IPPROTO_IP, IP_HDRINCL, (char *)&on, sizeof(on)) < 0)  {
        throw MyException (str_log(ERROR, "setsockopt"));
    }
}


PP_IcmpEcho::PP_IcmpEcho (uint32 destIP, uint16 icmp_id)
{
    log (INFO, "create PP_IcmpEcho");

    destIP_ = destIP;
    icmp_id_ = icmp_id;
    srcIP_ = destIP_ ? NetUtil::getRoute (destIP) : 0;
    
    if (destIP_) create_raw_socket (&sockd_);

    struct in_addr tmp1, tmp2;
    tmp1.s_addr = srcIP_; tmp2.s_addr = destIP_;

    log (INFO, "%s -> %s %u", string(inet_ntoa (tmp1)).c_str(), string(inet_ntoa (tmp2)).c_str(), icmp_id_);
}


PP_IcmpEcho::~PP_IcmpEcho ()
{
    if (destIP_) close (sockd_);
}


void
PP_IcmpEcho::
send (unsigned short seq)
{
    ASSERT (destIP_ != 0, "Cannot use send() to dummy probe (destIP==0)");
    log (INFO, "send probe seq = %u", seq);
    
    NetUtil::send_icmp_echoreq_packet (sockd_, srcIP_, destIP_, icmp_id_, seq);
}


bool
PP_IcmpEcho::
parse (char* rawIpPacket, int packet_len, uint16* seq)
{
    struct _iphdr* ip1;
    struct _icmphdr* icmp1;
    
    if (packet_len < sizeof(struct _iphdr) + sizeof(struct _icmphdr)) {
        log (INFO, "Bad packet. Too short.");
        return false;
    }
    ip1 = (struct _iphdr *)(rawIpPacket);
    icmp1 = (struct _icmphdr *)(rawIpPacket + sizeof(struct _iphdr));
    
    log (INFO, "%s", NetUtil::print_ip(ip1).c_str());
    
    if (ip1->protocol != IPPROTO_ICMP) {
        log (INFO, "Bad protocol. %d. Expect ICMP.", ip1->protocol);
        return false;
    }
    
    log (INFO, NetUtil::print_icmp (icmp1).c_str());
    
    if (icmp1->type == _ICMP_TYPE_ECHO_REPLY){
        log (INFO, "Got valid reply");
        *seq = ntohs(icmp1->un.echo.sequence);
        //*proc_delay = 0;
        return true;
    }
    
    return false;
}

string PP_IcmpEcho::info_str (void)
{
    ostringstream oss (ostringstream::out);
    
    oss << NetUtil::ip_uint32_to_str(srcIP_)
        << "->" << NetUtil::ip_uint32_to_str(destIP_) << ":" << icmp_id_
        << " icmp_echo";

    return oss.str();
}

