/*************************************************************/
/* iTraceroute (part of iSPY) Project                        */
/* Copyright (c) 2008, Zheng Zhang, Purdue University        */
/*************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <errno.h>

#include <iostream>
using namespace std;

#include "TF_MtBatch.h"
#include "Common.h"


TF_Mt::TF_Mt (int con_thrds)
{
    con_thrds_ = con_thrds;
}


void* _run_slave_thread (void* arg)
{
    int thrd_ith = ((TF_Mt::Slave_Thread_Arg*)arg)->thrd_ith;
    TF_Mt *me = ((TF_Mt::Slave_Thread_Arg*)arg)->this_ptr;
    me->slave_thread (thrd_ith);
    return NULL;
}


void TF_Mt::slave_thread (int thrd_ith)
{
    log(INFO, "Thread %d start exec", thrd_ith);

    while (! end_of_input ()) {      
        pthread_mutex_lock (&input_rlock);
        string cmd, tag;
        if (!get_input (&cmd, &tag)) {
            pthread_mutex_unlock (&input_rlock);
            continue;
        }
        pthread_mutex_unlock (&input_rlock);
        log (INFO, "Thread %d read in cmd %s", thrd_ith, cmd.c_str());

        string output;
        process_single_task (thrd_ith, cmd, &output);
        pthread_mutex_lock (&output_wlock);
        send_output (tag, cmd, output);
        log (INFO, "Thread %d output for cmd %s", thrd_ith, cmd.c_str());
        pthread_mutex_unlock (&output_wlock);
    }

    log(INFO, "Thread %d end exec", thrd_ith);
}


void TF_Mt::run (void)
{
    int res;
    res = pthread_mutex_init(&input_rlock, NULL);
    if (res != 0)
        throw MyException(str_log(ERROR, "Create a mutex"));
    res = pthread_mutex_init(&output_wlock, NULL);
    if (res != 0)
        throw MyException(str_log(ERROR, "Create a mutex"));        
     
    pthread_t* thread = new pthread_t[con_thrds_];
    Slave_Thread_Arg* thread_arg = new Slave_Thread_Arg[con_thrds_];
    
    for (int i = 0; i < con_thrds_; i ++) {
        thread_arg[i].thrd_ith = i;
        thread_arg[i].this_ptr = this;
        int res = pthread_create (&thread[i], NULL, _run_slave_thread, &thread_arg[i]);
        if (res != 0) throw MyException (str_log(ERROR, "Create a thread"));
    }
    
    log(INFO, "Waiting for children threads to finish");
    
    for (int i = 0; i < con_thrds_; i ++) {
        int res = pthread_join (thread[i], NULL);
        if (res != 0) throw MyException (str_log(ERROR, "Join a thread"));
        log(INFO, "Thread %d done", i);
    }
    
    log(INFO, "All children threads finished. Now clean up.");
    
    pthread_mutex_destroy(&input_rlock);
    pthread_mutex_destroy(&output_wlock);

    delete [] thread;
    delete [] thread_arg;

}


TF_MtFileBatch::TF_MtFileBatch (int con_thrds, ifstream* task_list_file)
    :TF_Mt (con_thrds)    
{
    task_list_file_ = task_list_file;
}


void TF_MtFileBatch::send_output (const string& tag, const string& cmd, const string& output)
{
    printf ("%s ", cmd.c_str());
    printf ("%s", output.c_str());
    printf ("\n"); //to be eye-friendly
    fflush (stdout); //Critical if some program is waiting for output
}


bool TF_MtFileBatch::get_input (string *cmd, string *tag)
{
    char line[512];
    task_list_file_->getline(line, 512);
    if (task_list_file_->bad())
        throw MyException (str_log(ERROR, "Error while reading input file"));
    if (task_list_file_->fail()) {
        return false;
    }
    
    *tag = string("");
    *cmd = string(line);
    return true;
}


bool TF_MtFileBatch::end_of_input ()
{
    return task_list_file_->eof ();
}

